@@ -82,9 +82,10 @@ use std::sync::atomic::{AtomicBool, Ordering};
 use std::sync::{Arc, Mutex, Weak};
 use std::thread;
 use std::time::Duration;
-use std::vec::Vec;
 #[cfg(feature = "ws-timed-fairness")]
-use time;
+use std::time::Instant;
+// use time;
+use std::vec::Vec;
 
 /// Creates a thread pool with support for up to 32 threads.
 ///
@@ -147,10 +148,10 @@ pub fn pool_with_auto_parker(threads: usize) -> ThreadPool<DynParker> {
 }
 
 #[cfg(feature = "ws-timed-fairness")]
-const CHECK_GLOBAL_INTERVAL_NS: u64 = timeconstants::NS_PER_MS;
+const CHECK_GLOBAL_INTERVAL_NS: u32 = timeconstants::NS_PER_MS;
 #[cfg(not(feature = "ws-timed-fairness"))]
 const CHECK_GLOBAL_MAX_MSGS: u64 = 100;
-const MAX_WAIT_SHUTDOWN_MS: u64 = 5 * timeconstants::MS_PER_S;
+const MAX_WAIT_SHUTDOWN_MS: u64 = 5 * (timeconstants::MS_PER_S as u64);
 
 // UnsafeCell has 10x the performance of RefCell
 // and the scoping guarantees that the borrows are exclusive
@@ -183,6 +184,12 @@ where
     /// This function will panic if `threads` is 0.
     /// It will also panic if `threads` is larger than the `ThreadData::MAX_THREADS` value of the provided `parker`.
     ///
+    /// # Core Affinity
+    ///
+    /// If compiled with `thread-pinning` it will assign a worker to each cores,
+    /// until it runs out of cores or workers. If there are more workers than cores,
+    /// the extra workers will be "floating", i.e. not pinned.
+    ///
     /// # Examples
     ///
     /// Create a new thread pool capable of executing four jobs concurrently:
@@ -208,12 +215,74 @@ where
             shutdown,
             parker,
         };
+        #[cfg(not(feature = "thread-pinning"))]
         {
             let mut guard = pool.core.lock().unwrap();
             for _ in 0..threads {
                 guard.spawn_worker(pool.core.clone(), None);
             }
         }
+        #[cfg(feature = "thread-pinning")]
+        {
+            let mut guard = pool.core.lock().unwrap();
+            let cores = core_affinity::get_core_ids().expect("core ids");
+            let num_pinned = cores.len().min(threads);
+            for i in 0..num_pinned {
+                guard.spawn_worker_pinned(pool.core.clone(), None, cores[i]);
+            }
+            if num_pinned < threads {
+                let num_unpinned = threads - num_pinned;
+                for _ in 0..num_unpinned {
+                    guard.spawn_worker(pool.core.clone(), None);
+                }
+            }
+        }
+        pool
+    }
+
+    /// Creates a new thread pool capable of executing `threads` number of jobs concurrently with a particular core affinity.
+    ///
+    /// For each core id in the `core` slice, it will generate a single thread pinned to that id.
+    /// Additionally, it will create `floating` number of unpinned threads.
+    ///
+    /// # Panics
+    ///
+    /// This function will panic if `cores.len() + floating` is 0.
+    #[cfg(feature = "thread-pinning")]
+    pub fn with_affinity(
+        cores: &[core_affinity::CoreId],
+        floating: usize,
+        parker: P,
+    ) -> ThreadPool<P> {
+        let total_threads = cores.len() + floating;
+        assert!(total_threads > 0);
+        if let Some(max) = parker.max_threads() {
+            assert!(total_threads <= max);
+        }
+        let injector = Arc::new(deque::Injector::new());
+        let shutdown = Arc::new(AtomicBool::new(false));
+        let core = ThreadPoolCore::new(
+            injector.clone(),
+            shutdown.clone(),
+            total_threads,
+            parker.clone(),
+        );
+        let pool = ThreadPool {
+            core: Arc::new(Mutex::new(core)),
+            global_sender: injector.clone(),
+            threads: total_threads,
+            shutdown,
+            parker,
+        };
+        {
+            let mut guard = pool.core.lock().unwrap();
+            cores.iter().for_each(|core_id| {
+                guard.spawn_worker_pinned(pool.core.clone(), None, *core_id);
+            });
+            for _ in 0..floating {
+                guard.spawn_worker(pool.core.clone(), None);
+            }
+        }
         pool
     }
 }
@@ -417,6 +486,32 @@ where
             })
             .expect("Could not create worker thread!");
     }
+
+    #[cfg(feature = "thread-pinning")]
+    fn spawn_worker_pinned(
+        &mut self,
+        core: Arc<Mutex<ThreadPoolCore<P>>>,
+        old_id: Option<usize>,
+        core_id: core_affinity::CoreId,
+    ) {
+        let id = old_id.unwrap_or_else(|| self.new_worker_id());
+        let (tx_control, rx_control) = channel::unbounded();
+        let worker = WorkerEntry {
+            control: tx_control,
+            stealer: Option::None,
+        };
+        let recv = self.global_injector.clone();
+        self.workers.insert(id, worker);
+        let p = self.parker.clone();
+        thread::Builder::new()
+            .name(format!("cb-ws-pool-worker-{}", id))
+            .spawn(move || {
+                core_affinity::set_for_current(core_id);
+                let mut worker = ThreadPoolWorker::new(id, recv, rx_control, core, p);
+                worker.run()
+            })
+            .expect("Could not create worker thread!");
+    }
 }
 
 impl<P> Drop for ThreadPoolCore<P>
@@ -516,7 +611,7 @@ where
         'main: loop {
             let mut fairness_check = false;
             #[cfg(feature = "ws-timed-fairness")]
-            let next_global_check = time::precise_time_ns() + CHECK_GLOBAL_INTERVAL_NS;
+            let next_global_check = Instant::now() + Duration::new(0, CHECK_GLOBAL_INTERVAL_NS);
             // Try the local queue usually
             LOCAL_JOB_QUEUE.with(|q| unsafe {
                 if let Some(ref local_queue) = *q.get() {
@@ -541,7 +636,7 @@ where
                         }
                         #[cfg(feature = "ws-timed-fairness")]
                         {
-                            if (time::precise_time_ns() > next_global_check) {
+                            if (Instant::now() > next_global_check) {
                                 fairness_check = true;
                                 break 'local;
                             }
@@ -830,6 +925,16 @@ mod tests {
         crate::tests::test_custom(exec, LABEL);
     }
 
+    #[cfg(feature = "thread-pinning")]
+    #[test]
+    fn test_custom_affinity() {
+        let cores = core_affinity::get_core_ids().expect("core ids");
+        // travis doesn't have enough cores for this
+        //let exec = ThreadPool::with_affinity(&cores[0..4], 4, parker::small());
+        let exec = ThreadPool::with_affinity(&cores[0..1], 1, parker::small());
+        crate::tests::test_custom(exec, LABEL);
+    }
+
     #[test]
     fn test_defaults() {
         crate::tests::test_defaults::<ThreadPool<DynParker>>(LABEL);
